"""
Moduł bazowy konwertera definiujący interfejs dla wszystkich konwerterów.
"""

from abc import ABC, abstractmethod
from pathlib import Path
from typing import Any, Dict, List, Optional, Type, Union


class BaseConverter(ABC):
    """
    Klasa bazowa dla wszystkich konwerterów formatów pakietów.

    Każdy konwerter musi implementować metody read_source, convert, i write_target.
    """

    def __init__(
        self,
        source_file: Optional[Union[str, Path]] = None,
        target_file: Optional[Union[str, Path]] = None,
        options: Optional[Dict[str, Any]] = None,
    ):
        """
        Inicjalizacja konwertera.

        Args:
            source_file: Ścieżka do pliku źródłowego
            target_file: Ścieżka do pliku docelowego
            options: Dodatkowe opcje konfiguracyjne
        """
        self.source_file = Path(source_file) if source_file else None
        self.target_file = Path(target_file) if target_file else None
        self.options = options or {}
        self.source_data = None
        self.target_data = None

    @property
    def source_format(self) -> str:
        """Format źródłowy obsługiwany przez konwerter."""
        return self.get_source_format()

    @property
    def target_format(self) -> str:
        """Format docelowy obsługiwany przez konwerter."""
        return self.get_target_format()

    @staticmethod
    @abstractmethod
    def get_source_format() -> str:
        """Zwraca identyfikator formatu źródłowego."""
        pass

    @staticmethod
    @abstractmethod
    def get_target_format() -> str:
        """Zwraca identyfikator formatu docelowego."""
        pass

    @abstractmethod
    def read_source(self) -> Dict[str, Any]:
        """
        Odczytuje dane z pliku źródłowego.

        Returns:
            Odczytane dane w ustandaryzowanym formacie słownikowym
        """
        pass

    @abstractmethod
    def convert(self, source_data: Optional[Dict[str, Any]] = None) -> Dict[str, Any]:
        """
        Konwertuje dane ze źródłowego formatu na docelowy.

        Args:
            source_data: Dane w formacie źródłowym (opcjonalnie, jeśli nie podano używa self.source_data)

        Returns:
            Przekonwertowane dane w formacie docelowym
        """
        pass

    @abstractmethod
    def write_target(self, target_data: Optional[Dict[str, Any]] = None) -> Path:
        """
        Zapisuje przekonwertowane dane do pliku docelowego.

        Args:
            target_data: Dane w formacie docelowym (opcjonalnie, jeśli nie podano używa self.target_data)

        Returns:
            Ścieżka do zapisanego pliku
        """
        pass

    def execute(self) -> Path:
        """
        Wykonuje pełny proces konwersji: odczyt, konwersja, zapis.

        Returns:
            Ścieżka do zapisanego pliku docelowego
        """
        self.source_data = self.read_source()
        self.target_data = self.convert(self.source_data)
        return self.write_target(self.target_data)



"""
Spectomate - modularny konwerter pakietów Python

Narzędzie do konwersji między różnymi formatami zarządzania pakietami w Pythonie.
"""

__version__ = "0.1.28"
__author__ = "Tom Sapletta"

from spectomate.core.base_converter import BaseConverter
from spectomate.core.registry import ConverterRegistry

# Inicjalizacja rejestru konwerterów
registry = ConverterRegistry()

# Importy dla wygodnego dostępu do API
from spectomate.converters import *  # noqa



"""
Konwerter z formatu conda (environment.yml) do formatu pip (requirements.txt).
"""

import os
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from spectomate.core.base_converter import BaseConverter
from spectomate.schemas.conda_schema import CondaSchema
from spectomate.schemas.pip_schema import PipSchema


class CondaToPipConverter(BaseConverter):
    """
    Konwerter z formatu conda (environment.yml) do formatu pip (requirements.txt).
    """

    def __init__(
        self,
        source_file: Optional[Union[str, Path]] = None,
        target_file: Optional[Union[str, Path]] = None,
        options: Optional[Dict[str, Any]] = None,
    ):
        """
        Inicjalizuje konwerter.

        Args:
            source_file: Ścieżka do pliku environment.yml
            target_file: Ścieżka do pliku requirements.txt
            options: Opcje konwersji
        """
        super().__init__(source_file, target_file, options)

        # Ustawiamy domyślne opcje
        if self.options is None:
            self.options = {}

    @staticmethod
    def get_source_format() -> str:
        """Zwraca identyfikator formatu źródłowego."""
        return "conda"

    @staticmethod
    def get_target_format() -> str:
        """Zwraca identyfikator formatu docelowego."""
        return "pip"

    def read_source(self) -> Dict[str, Any]:
        """
        Odczytuje plik environment.yml.

        Returns:
            Słownik z informacjami o środowisku conda
        """
        if self.source_file is None:
            raise ValueError("Nie podano ścieżki do pliku źródłowego")

        return CondaSchema.parse_file(self.source_file)

    def convert(self, source_data: Dict[str, Any]) -> Dict[str, Any]:
        """
        Konwertuje dane z formatu conda do formatu pip.

        Args:
            source_data: Dane w formacie conda

        Returns:
            Dane w formacie pip
        """
        # Inicjalizujemy dane wyjściowe
        target_data = {
            "format": "pip",
            "requirements": [],  # Używamy klucza "requirements" zamiast "dependencies"
        }

        # Pobieramy zależności conda
        conda_deps = CondaSchema.extract_conda_dependencies(source_data)

        # Pobieramy zależności pip
        pip_deps = CondaSchema.extract_pip_dependencies(source_data)

        # Konwertujemy zależności conda na format pip
        for dep in conda_deps:
            # Usuwamy specyfikację kanału, jeśli istnieje
            if "::" in dep:
                _, dep = dep.split("::", 1)

            # Dodajemy zależność do listy
            target_data["requirements"].append(dep)

        # Dodajemy zależności pip
        target_data["requirements"].extend(pip_deps)

        # Sortujemy zależności
        target_data["requirements"].sort()

        return target_data

    def write_target(self, target_data: Dict[str, Any]) -> Path:
        """
        Zapisuje dane do pliku requirements.txt.

        Args:
            target_data: Dane w formacie pip

        Returns:
            Ścieżka do zapisanego pliku
        """
        if self.target_file is None:
            # Jeśli nie podano ścieżki do pliku docelowego, tworzymy ją na podstawie pliku źródłowego
            if self.source_file is None:
                raise ValueError(
                    "Nie podano ścieżki do pliku docelowego ani źródłowego"
                )

            source_path = Path(self.source_file)
            self.target_file = source_path.parent / "requirements.txt"

        return PipSchema.write_requirements_txt(target_data, self.target_file)

    def execute(self) -> Path:
        """
        Wykonuje pełny proces konwersji.

        Returns:
            Ścieżka do zapisanego pliku
        """
        # Odczytujemy plik źródłowy
        source_data = self.read_source()

        # Konwertujemy dane
        target_data = self.convert(source_data)

        # Zapisujemy dane do pliku docelowego
        return self.write_target(target_data)



"""
Command-line interface for mypy helper tools.

This module provides CLI commands to help users identify and fix mypy issues.
"""

import os
from pathlib import Path
from typing import Optional

import click
from rich.console import Console

from spectomate.core.mypy_helper import (
    add_type_ignore_comments,
    apply_monkeytype,
    create_mypy_config,
    display_mypy_issues,
    fix_common_mypy_issues,
    generate_stubs_for_module,
    run_mypy,
)

# Initialize rich console
console = Console()


@click.group()
def mypy_cli():
    """Tools to help with mypy type checking issues."""
    pass


@mypy_cli.command("check")
@click.argument("path", type=click.Path(exists=True))
@click.option(
    "--config", "-c", type=click.Path(exists=True), help="Path to mypy config file"
)
@click.option("--strict", is_flag=True, help="Run mypy in strict mode")
@click.option("--verbose", "-v", is_flag=True, help="Show verbose output")
@click.option(
    "--ignore-missing-imports/--no-ignore-missing-imports",
    default=True,
    help="Ignore missing imports (default: True)",
)
def check_command(
    path: str,
    config: Optional[str],
    strict: bool,
    verbose: bool,
    ignore_missing_imports: bool,
):
    """Run mypy and display issues with suggested fixes."""
    success, issues = run_mypy(
        target_path=path,
        config_file=config,
        strict=strict,
        verbose=verbose,
        ignore_missing_imports=ignore_missing_imports,
    )

    if success:
        console.print("[green]No mypy issues found![/green]")
    else:
        display_mypy_issues(issues, verbose=verbose)


@mypy_cli.command("fix")
@click.argument("path", type=click.Path(exists=True))
@click.option("--auto", is_flag=True, help="Automatically apply fixes where possible")
@click.option(
    "--interactive", "-i", is_flag=True, help="Interactively prompt for fixes"
)
@click.option("--verbose", "-v", is_flag=True, help="Show verbose output")
def fix_command(path: str, auto: bool, interactive: bool, verbose: bool):
    """Attempt to fix common mypy issues."""
    if not auto and not interactive:
        interactive = True  # Default to interactive mode if neither is specified

    fixed = fix_common_mypy_issues(
        target_path=path, auto_fix=auto, interactive=interactive, verbose=verbose
    )

    if fixed:
        console.print("[green]All mypy issues fixed![/green]")
    else:
        console.print(
            "[yellow]Some mypy issues could not be fixed automatically.[/yellow]"
        )


@mypy_cli.command("ignore")
@click.argument("path", type=click.Path(exists=True))
@click.option(
    "--selective",
    "-s",
    is_flag=True,
    help="Only add ignores for certain types of issues",
)
@click.option("--verbose", "-v", is_flag=True, help="Show verbose output")
def ignore_command(path: str, selective: bool, verbose: bool):
    """Add # type: ignore comments to lines with mypy issues."""
    # First run mypy to get issues
    success, issues = run_mypy(target_path=path, verbose=verbose)

    if success:
        console.print("[green]No mypy issues found![/green]")
        return

    # Add type: ignore comments
    comments_added = add_type_ignore_comments(
        target_path=path, issues=issues, selective=selective
    )

    console.print(f"[green]Added {comments_added} type: ignore comments[/green]")


@mypy_cli.command("stubs")
@click.argument("module_name")
@click.option("--output", "-o", type=click.Path(), help="Output directory for stubs")
def stubs_command(module_name: str, output: Optional[str]):
    """Generate stub files for a module."""
    output_path = Path(output) if output else None
    success = generate_stubs_for_module(module_name, output_path)

    if success:
        console.print(f"[green]Successfully generated stubs for {module_name}[/green]")
    else:
        console.print(f"[red]Failed to generate stubs for {module_name}[/red]")


@mypy_cli.command("monkeytype")
@click.argument("script_path", type=click.Path(exists=True))
@click.option("--module", "-m", help="Module name to apply types to")
@click.option("--apply", is_flag=True, help="Apply the generated types")
def monkeytype_command(script_path: str, module: Optional[str], apply: bool):
    """Run monkeytype on a script to generate type annotations."""
    success = apply_monkeytype(script_path=script_path, module_name=module, apply=apply)

    if success and not apply:
        console.print("[green]Successfully ran script with monkeytype[/green]")
        console.print("To apply the generated types, run:")
        console.print(
            f"  spectomate mypy monkeytype {script_path} --module {module or 'MODULE_NAME'} --apply"
        )


@mypy_cli.command("config")
@click.argument("directory", type=click.Path(exists=True, file_okay=False))
@click.option("--strict", is_flag=True, help="Enable strict mode")
@click.option(
    "--ignore-missing-imports",
    is_flag=True,
    default=True,
    help="Ignore missing imports",
)
@click.option(
    "--disallow-untyped-defs",
    is_flag=True,
    help="Disallow untyped function definitions",
)
@click.option(
    "--disallow-incomplete-defs",
    is_flag=True,
    help="Disallow incomplete function definitions",
)
def config_command(
    directory: str,
    strict: bool,
    ignore_missing_imports: bool,
    disallow_untyped_defs: bool,
    disallow_incomplete_defs: bool,
):
    """Create a mypy configuration file."""
    config_path = create_mypy_config(
        target_dir=directory,
        strict=strict,
        ignore_missing_imports=ignore_missing_imports,
        disallow_untyped_defs=disallow_untyped_defs,
        disallow_incomplete_defs=disallow_incomplete_defs,
    )

    console.print(f"[green]Created mypy config file at {config_path}[/green]")


if __name__ == "__main__":
    mypy_cli()



"""
Schemat dla formatu poetry (pyproject.toml).
"""

from pathlib import Path
from typing import Any, Dict, List, Optional, Set, Union

import toml


class PoetrySchema:
    """
    Klasa definiująca schemat dla formatu poetry (pyproject.toml).
    """

    @staticmethod
    def parse_file(file_path: Union[str, Path]) -> Dict[str, Any]:
        """
        Parsuje plik pyproject.toml.

        Args:
            file_path: Ścieżka do pliku pyproject.toml

        Returns:
            Słownik z informacjami o projekcie poetry
        """
        file_path = Path(file_path)

        if not file_path.exists():
            raise FileNotFoundError(f"Plik nie istnieje: {file_path}")

        try:
            pyproject_data = toml.load(file_path)
        except Exception as e:
            raise ValueError(f"Błąd parsowania pliku TOML: {e}")

        # Sprawdzamy, czy plik ma sekcję [tool.poetry]
        if "tool" not in pyproject_data or "poetry" not in pyproject_data["tool"]:
            raise ValueError("Plik pyproject.toml nie zawiera sekcji [tool.poetry]")

        # Pobieramy sekcję poetry
        poetry_data = pyproject_data["tool"]["poetry"]

        # Dodajemy domyślne wartości, jeśli brakuje kluczy
        if "name" not in poetry_data:
            poetry_data["name"] = "myproject"

        if "version" not in poetry_data:
            poetry_data["version"] = "0.1.0"

        if "description" not in poetry_data:
            poetry_data["description"] = ""

        if "dependencies" not in poetry_data:
            poetry_data["dependencies"] = {}

        if "dev-dependencies" not in poetry_data and "group" not in poetry_data:
            poetry_data["dev-dependencies"] = {}

        # Dodajemy informację o formacie
        poetry_data["format"] = "poetry"

        return poetry_data

    @staticmethod
    def extract_dependencies(
        poetry_data: Dict[str, Any], include_dev: bool = False
    ) -> List[str]:
        """
        Wyodrębnia zależności z danych poetry.

        Args:
            poetry_data: Słownik z informacjami o projekcie poetry
            include_dev: Czy uwzględnić zależności deweloperskie

        Returns:
            Lista zależności w formacie pip
        """
        dependencies = []

        # Pobieramy zależności główne
        main_deps = poetry_data.get("dependencies", {})
        for name, constraint in main_deps.items():
            # Pomijamy python jako zależność
            if name.lower() == "python":
                continue

            if isinstance(constraint, str):
                # Prosty przypadek: "package = "version""
                dependencies.append(f"{name}{constraint}")
            elif isinstance(constraint, dict):
                # Złożony przypadek: "package = {version = "version", extras = [...]}"
                if "version" in constraint:
                    version = constraint["version"]
                    if "extras" in constraint:
                        extras = ",".join(constraint["extras"])
                        dependencies.append(f"{name}[{extras}]{version}")
                    else:
                        dependencies.append(f"{name}{version}")
                elif "git" in constraint:
                    # Zależność z git
                    git_url = constraint["git"]
                    if "rev" in constraint:
                        rev = constraint["rev"]
                        dependencies.append(f"git+{git_url}@{rev}#egg={name}")
                    else:
                        dependencies.append(f"git+{git_url}#egg={name}")
                elif "url" in constraint:
                    # Zależność z URL
                    url = constraint["url"]
                    dependencies.append(f"{name} @ {url}")
            else:
                # Nieznany format, dodajemy samą nazwę
                dependencies.append(name)

        # Pobieramy zależności deweloperskie, jeśli wymagane
        if include_dev:
            # Sprawdzamy, czy używamy starego formatu (dev-dependencies) czy nowego (group.dev.dependencies)
            if "dev-dependencies" in poetry_data:
                dev_deps = poetry_data.get("dev-dependencies", {})
                for name, constraint in dev_deps.items():
                    if isinstance(constraint, str):
                        dependencies.append(f"{name}{constraint}")
                    elif isinstance(constraint, dict) and "version" in constraint:
                        dependencies.append(f"{name}{constraint['version']}")
                    else:
                        dependencies.append(name)
            elif "group" in poetry_data and "dev" in poetry_data["group"]:
                dev_deps = poetry_data["group"]["dev"].get("dependencies", {})
                for name, constraint in dev_deps.items():
                    if isinstance(constraint, str):
                        dependencies.append(f"{name}{constraint}")
                    elif isinstance(constraint, dict) and "version" in constraint:
                        dependencies.append(f"{name}{constraint['version']}")
                    else:
                        dependencies.append(name)

        return dependencies

    @staticmethod
    def generate_pyproject_toml(data: Dict[str, Any]) -> str:
        """
        Generuje zawartość pliku pyproject.toml na podstawie danych.

        Args:
            data: Dane w formacie schematu poetry

        Returns:
            Zawartość pliku pyproject.toml
        """
        # Tworzymy kopię danych, aby nie modyfikować oryginału
        poetry_data = data.copy()

        # Usuwamy pole format, które nie jest częścią standardowego pliku pyproject.toml
        if "format" in poetry_data:
            del poetry_data["format"]

        # Tworzymy strukturę pyproject.toml
        pyproject = {
            "tool": {"poetry": poetry_data},
            "build-system": {
                "requires": ["poetry-core>=1.0.0"],
                "build-backend": "poetry.core.masonry.api",
            },
        }

        # Konwertujemy dane do TOML
        return toml.dumps(pyproject)

    @staticmethod
    def write_pyproject_toml(
        data: Dict[str, Any], output_path: Union[str, Path]
    ) -> Path:
        """
        Zapisuje dane do pliku pyproject.toml.

        Args:
            data: Dane w formacie schematu poetry
            output_path: Ścieżka do pliku wyjściowego

        Returns:
            Ścieżka do zapisanego pliku
        """
        output_path = Path(output_path)

        # Tworzymy katalogi, jeśli nie istnieją
        output_path.parent.mkdir(parents=True, exist_ok=True)

        content = PoetrySchema.generate_pyproject_toml(data)

        with open(output_path, "w") as f:
            f.write(content)

        return output_path

    @staticmethod
    def convert_from_pip(
        pip_data: Dict[str, Any],
        project_name: str = "myproject",
        version: str = "0.1.0",
    ) -> Dict[str, Any]:
        """
        Konwertuje dane z formatu pip do formatu poetry.

        Args:
            pip_data: Dane w formacie pip
            project_name: Nazwa projektu
            version: Wersja projektu

        Returns:
            Dane w formacie poetry
        """
        # Tworzymy podstawową strukturę danych poetry
        poetry_data = {
            "name": project_name,
            "version": version,
            "description": "",
            "authors": ["Your Name <your.email@example.com>"],
            "readme": "README.md",
            "dependencies": {},
            "dev-dependencies": {},
            "format": "poetry",
        }

        # Pobieramy zależności - obsługujemy zarówno klucz "dependencies" jak i "requirements"
        dependencies = pip_data.get("requirements", pip_data.get("dependencies", []))

        # Konwertujemy zależności
        for dep in dependencies:
            # Obsługa przypadku, gdy dep jest słownikiem (rozszerzony format zależności)
            if isinstance(dep, dict):
                if dep.get("type") == "package":
                    package_name = dep["name"]

                    # Sprawdzamy, czy jest specyfikacja wersji
                    if "version_spec" in dep:
                        operator = dep["version_spec"]["operator"]
                        version_val = dep["version_spec"]["version"]
                        poetry_data["dependencies"][
                            package_name
                        ] = f"{operator}{version_val}"
                    else:
                        poetry_data["dependencies"][package_name] = "*"

                # Pomijamy komentarze i opcje
                continue

            # Obsługa przypadku, gdy dep jest stringiem (prosty format zależności)
            # Pomijamy komentarze
            if isinstance(dep, str):
                if dep.startswith("#"):
                    continue

                # Parsujemy zależność
                if "==" in dep:
                    name, version_val = dep.split("==", 1)
                    poetry_data["dependencies"][
                        name.strip()
                    ] = f"=={version_val.strip()}"
                elif ">=" in dep:
                    name, version_val = dep.split(">=", 1)
                    poetry_data["dependencies"][
                        name.strip()
                    ] = f">={version_val.strip()}"
                elif "<=" in dep:
                    name, version_val = dep.split("<=", 1)
                    poetry_data["dependencies"][
                        name.strip()
                    ] = f"<={version_val.strip()}"
                elif ">" in dep:
                    name, version_val = dep.split(">", 1)
                    poetry_data["dependencies"][
                        name.strip()
                    ] = f">{version_val.strip()}"
                elif "<" in dep:
                    name, version_val = dep.split("<", 1)
                    poetry_data["dependencies"][
                        name.strip()
                    ] = f"<{version_val.strip()}"
                elif "~=" in dep:
                    name, version_val = dep.split("~=", 1)
                    poetry_data["dependencies"][
                        name.strip()
                    ] = f"~{version_val.strip()}"
                elif "@" in dep:
                    # URL dependency
                    name, url = dep.split("@", 1)
                    poetry_data["dependencies"][name.strip()] = {"url": url.strip()}
                elif dep.startswith("git+"):
                    # Git dependency
                    url = dep[4:]
                    if "#egg=" in url:
                        url, egg = url.split("#egg=", 1)
                        name = egg.strip()
                        if "@" in url:
                            url, rev = url.split("@", 1)
                            poetry_data["dependencies"][name] = {"git": url, "rev": rev}
                        else:
                            poetry_data["dependencies"][name] = {"git": url}
                    else:
                        # Nieznany format git, dodajemy jako string
                        poetry_data["dependencies"][dep] = "*"
                else:
                    # Prosta zależność bez wersji
                    poetry_data["dependencies"][dep.strip()] = "*"

        return poetry_data



"""
Schemat dla formatu pip (requirements.txt).
"""

import re
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple, Union


class PipSchema:
    """
    Klasa definiująca schemat dla formatu pip (requirements.txt).
    """

    @staticmethod
    def parse_requirement(req_line: str) -> Dict[str, Any]:
        """
        Parsuje linię z pliku requirements.txt.

        Args:
            req_line: Linia z pliku requirements.txt

        Returns:
            Słownik z informacjami o zależności
        """
        # Usuwamy komentarze i białe znaki
        line = req_line.split("#")[0].strip()

        if not line or line.startswith("#"):
            return {"type": "comment", "content": req_line}

        if line.startswith("-"):
            return {"type": "option", "content": line}

        # Obsługa specyfikacji wersji
        # Przykłady: package==1.0.0, package>=1.0.0, package<=1.0.0
        package_pattern = (
            r"([a-zA-Z0-9_\-\.]+)(?:\s*([<>=!~]{1,2})\s*([a-zA-Z0-9_\-\.]+))?"
        )
        match = re.match(package_pattern, line)

        if match:
            package_name = match.group(1)
            operator = match.group(2)
            version = match.group(3)

            result = {
                "type": "package",
                "name": package_name,
            }

            if operator and version:
                result["version_spec"] = {"operator": operator, "version": version}

            return result

        # Jeśli nie pasuje do żadnego wzorca, traktujemy jako prosty pakiet
        return {"type": "package", "name": line}

    @staticmethod
    def parse_file(file_path: Union[str, Path]) -> Dict[str, Any]:
        """
        Parsuje plik requirements.txt.

        Args:
            file_path: Ścieżka do pliku requirements.txt

        Returns:
            Słownik z listą zależności
        """
        file_path = Path(file_path)

        if not file_path.exists():
            raise FileNotFoundError(f"Plik nie istnieje: {file_path}")

        requirements = []

        with open(file_path, "r") as f:
            for line in f:
                line = line.strip()
                if line:  # Pomijamy puste linie
                    req = PipSchema.parse_requirement(line)
                    requirements.append(req)

        return {"format": "pip", "requirements": requirements}

    @staticmethod
    def generate_requirements_txt(data: Dict[str, Any]) -> str:
        """
        Generuje zawartość pliku requirements.txt na podstawie danych.

        Args:
            data: Dane w formacie schematu pip

        Returns:
            Zawartość pliku requirements.txt
        """
        if "requirements" not in data:
            raise ValueError("Brak wymaganych zależności w danych")

        lines = []

        for req in data["requirements"]:
            # Obsługa przypadku, gdy req jest stringiem (prosty format zależności)
            if isinstance(req, str):
                lines.append(req)
                continue

            # Obsługa przypadku, gdy req jest słownikiem (rozszerzony format zależności)
            req_type = req.get("type")

            if req_type == "comment":
                lines.append(req["content"])

            elif req_type == "option":
                lines.append(req["content"])

            elif req_type == "package":
                package_line = req["name"]

                if "version_spec" in req:
                    spec = req["version_spec"]
                    package_line += f"{spec['operator']}{spec['version']}"

                lines.append(package_line)

        return "\n".join(lines)

    @staticmethod
    def write_requirements_txt(
        data: Dict[str, Any], output_path: Union[str, Path]
    ) -> Path:
        """
        Zapisuje dane do pliku requirements.txt.

        Args:
            data: Dane w formacie schematu pip
            output_path: Ścieżka do pliku wyjściowego

        Returns:
            Ścieżka do zapisanego pliku
        """
        output_path = Path(output_path)

        # Tworzymy katalogi, jeśli nie istnieją
        output_path.parent.mkdir(parents=True, exist_ok=True)

        content = PipSchema.generate_requirements_txt(data)

        with open(output_path, "w") as f:
            f.write(content)

        return output_path



"""
Rejestr konwerterów umożliwiający dynamiczne rejestrowanie i odnajdywanie dostępnych konwerterów.
"""

from typing import Dict, List, Optional, Set, Tuple, Type

from spectomate.core.base_converter import BaseConverter


class ConverterRegistry:
    """
    Rejestr konwerterów zarządzający dostępnymi konwerterami.
    """

    _converters: Dict[Tuple[str, str], Type[BaseConverter]] = {}

    @classmethod
    def register(cls, converter_class: Type[BaseConverter]) -> None:
        """
        Rejestruje klasę konwertera w rejestrze.

        Args:
            converter_class: Klasa konwertera do zarejestrowania
        """
        source_format = converter_class.get_source_format()
        target_format = converter_class.get_target_format()
        key = (source_format, target_format)

        cls._converters[key] = converter_class

    @classmethod
    def get_converter(
        cls, source_format: str, target_format: str
    ) -> Optional[Type[BaseConverter]]:
        """
        Zwraca klasę konwertera dla podanej pary formatów.

        Args:
            source_format: Format źródłowy
            target_format: Format docelowy

        Returns:
            Klasa konwertera lub None jeśli nie znaleziono
        """
        key = (source_format, target_format)
        return cls._converters.get(key)

    @classmethod
    def get_converters(
        cls, input_format: Optional[str] = None, output_format: Optional[str] = None
    ) -> List[Type[BaseConverter]]:
        """
        Zwraca listę wszystkich dostępnych konwerterów, opcjonalnie filtrowanych według formatu.

        Args:
            input_format: Opcjonalny filtr formatu wejściowego
            output_format: Opcjonalny filtr formatu wyjściowego

        Returns:
            Lista konwerterów spełniających kryteria
        """
        converters = []

        for (source, target), converter_class in cls._converters.items():
            if input_format is not None and source != input_format:
                continue
            if output_format is not None and target != output_format:
                continue
            converters.append(converter_class)

        return converters

    @classmethod
    def get_source_formats(cls) -> Set[str]:
        """
        Zwraca zbiór wszystkich dostępnych formatów źródłowych.

        Returns:
            Zbiór nazw formatów źródłowych
        """
        return {source for source, _ in cls._converters.keys()}

    @classmethod
    def get_target_formats(cls) -> Set[str]:
        """
        Zwraca zbiór wszystkich dostępnych formatów docelowych.

        Returns:
            Zbiór nazw formatów docelowych
        """
        return {target for _, target in cls._converters.keys()}

    @classmethod
    def get_all_formats(cls) -> Set[str]:
        """
        Zwraca zbiór wszystkich dostępnych formatów (źródłowych i docelowych).

        Returns:
            Zbiór nazw formatów
        """
        sources = cls.get_source_formats()
        targets = cls.get_target_formats()
        return sources.union(targets)

    @classmethod
    def get_input_formats(cls) -> Set[str]:
        """
        Zwraca zbiór wszystkich dostępnych formatów wejściowych.

        Alias dla get_source_formats().

        Returns:
            Zbiór nazw formatów wejściowych
        """
        return cls.get_source_formats()

    @classmethod
    def get_output_formats(cls) -> Set[str]:
        """
        Zwraca zbiór wszystkich dostępnych formatów wyjściowych.

        Alias dla get_target_formats().

        Returns:
            Zbiór nazw formatów wyjściowych
        """
        return cls.get_target_formats()

    @classmethod
    def has_converter(cls, source_format: str, target_format: str) -> bool:
        """
        Sprawdza, czy istnieje konwerter dla podanej pary formatów.

        Args:
            source_format: Format źródłowy
            target_format: Format docelowy

        Returns:
            True jeśli konwerter istnieje, False w przeciwnym wypadku
        """
        return cls.get_converter(source_format, target_format) is not None


def register_converter(converter_class: Type[BaseConverter]) -> Type[BaseConverter]:
    """
    Dekorator do rejestrowania konwerterów.

    Args:
        converter_class: Klasa konwertera do zarejestrowania

    Returns:
        Niezmieniona klasa konwertera
    """
    ConverterRegistry.register(converter_class)
    return converter_class



"""
Interfejs wiersza poleceń dla Spectomate.
"""

import os
import sys
from pathlib import Path
from typing import Dict, List, Optional, Set

import click

from spectomate import __version__, registry
from spectomate.core.utils import get_available_formats
from spectomate.format_cli import format_cli
from spectomate.git_cli import git_cli
from spectomate.mypy_cli import mypy_cli
from spectomate.test_cli import test_cli
from spectomate.update_cli import update_command


@click.group(invoke_without_command=True)
@click.option("--version", is_flag=True, help="Show version and exit")
@click.pass_context
def cli(ctx: click.Context, version: bool):
    """Spectomate - modularny konwerter formatów pakietów Python."""
    if version:
        click.echo(f"Spectomate version: {__version__}")
        ctx.exit()

    if ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())


@cli.command()
@click.option(
    "--input-format",
    "-i",
    help="Format wejściowy",
    type=click.Choice(get_available_formats("input")),
    required=True,
)
@click.option(
    "--output-format",
    "-o",
    help="Format wyjściowy",
    type=click.Choice(get_available_formats("output")),
    required=True,
)
@click.option(
    "--input-file",
    "-f",
    help="Plik wejściowy",
    type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True),
    required=True,
)
@click.option(
    "--output-file",
    "-t",
    help="Plik wyjściowy",
    type=click.Path(file_okay=True, dir_okay=False, writable=True),
    required=True,
)
def convert(input_format: str, output_format: str, input_file: str, output_file: str):
    """Konwertuje plik z jednego formatu na drugi."""
    # Sprawdź, czy istnieje konwerter dla podanej pary formatów
    if not registry.has_converter(input_format, output_format):
        click.echo(
            f"Nie znaleziono konwertera z formatu {input_format} do {output_format}",
            err=True,
        )
        sys.exit(1)

    # Pobierz konwerter
    converter = registry.get_converter(input_format, output_format)

    # Konwertuj plik
    try:
        converter.convert_file(input_file, output_file)
        click.echo(f"Pomyślnie skonwertowano {input_file} do {output_file}")
    except Exception as e:
        click.echo(f"Błąd podczas konwersji: {e}", err=True)
        sys.exit(1)


@cli.command()
@click.option(
    "--input-format",
    "-i",
    help="Format wejściowy",
    type=click.Choice(get_available_formats("input")),
    required=False,
)
@click.option(
    "--output-format",
    "-o",
    help="Format wyjściowy",
    type=click.Choice(get_available_formats("output")),
    required=False,
)
def list_converters(input_format: Optional[str], output_format: Optional[str]):
    """Wyświetla listę dostępnych konwerterów."""
    converters = registry.get_converters(input_format, output_format)

    if not converters:
        if input_format and output_format:
            click.echo(
                f"Nie znaleziono konwerterów z formatu {input_format} do {output_format}",
                err=True,
            )
        elif input_format:
            click.echo(f"Nie znaleziono konwerterów z formatu {input_format}", err=True)
        elif output_format:
            click.echo(
                f"Nie znaleziono konwerterów do formatu {output_format}", err=True
            )
        else:
            click.echo("Nie znaleziono żadnych konwerterów", err=True)
        sys.exit(1)

    click.echo("Dostępne konwertery:")
    for converter in converters:
        click.echo(
            f"  {converter.input_format} -> {converter.output_format}: {converter.description}"
        )


# Dodaj komendy z innych modułów
cli.add_command(update_command)
cli.add_command(test_cli)
cli.add_command(mypy_cli)
cli.add_command(git_cli)
cli.add_command(format_cli)


def main():
    """Entry point for the CLI."""
    return cli()


if __name__ == "__main__":
    sys.exit(main())



"""
Konwerter z formatu pip (requirements.txt) do formatu poetry (pyproject.toml).
"""

import os
from pathlib import Path
from typing import Any, Dict, List, Optional, Union

from spectomate.core.base_converter import BaseConverter
from spectomate.schemas.pip_schema import PipSchema
from spectomate.schemas.poetry_schema import PoetrySchema


class PipToPoetryConverter(BaseConverter):
    """
    Konwerter z formatu pip (requirements.txt) do formatu poetry (pyproject.toml).
    """

    def __init__(
        self,
        source_file: Optional[Union[str, Path]] = None,
        target_file: Optional[Union[str, Path]] = None,
        options: Optional[Dict[str, Any]] = None,
    ):
        """
        Inicjalizuje konwerter.

        Args:
            source_file: Ścieżka do pliku requirements.txt
            target_file: Ścieżka do pliku pyproject.toml
            options: Opcje konwersji
        """
        super().__init__(source_file, target_file, options)

        # Ustawiamy domyślne opcje
        if self.options is None:
            self.options = {}

    @staticmethod
    def get_source_format() -> str:
        """Zwraca identyfikator formatu źródłowego."""
        return "pip"

    @staticmethod
    def get_target_format() -> str:
        """Zwraca identyfikator formatu docelowego."""
        return "poetry"

    def read_source(self) -> Dict[str, Any]:
        """
        Odczytuje plik requirements.txt.

        Returns:
            Słownik z zależnościami pip
        """
        if self.source_file is None:
            raise ValueError("Nie podano ścieżki do pliku źródłowego")

        data = PipSchema.parse_file(self.source_file)

        # Filtrujemy tylko rzeczywiste zależności (pakiety) i pomijamy komentarze i opcje
        if "requirements" in data:
            package_deps = [
                dep
                for dep in data["requirements"]
                if isinstance(dep, dict) and dep.get("type") == "package"
            ]

            # Konwertujemy słowniki pakietów na stringi dla testów
            dependencies = []
            for dep in package_deps:
                if "version_spec" in dep:
                    dependencies.append(
                        f"{dep['name']}{dep['version_spec']['operator']}{dep['version_spec']['version']}"
                    )
                else:
                    dependencies.append(dep["name"])

            # Ustawiamy klucz "dependencies" dla zachowania kompatybilności z testami
            data["dependencies"] = dependencies

        return data

    def convert(self, source_data: Dict[str, Any]) -> Dict[str, Any]:
        """
        Konwertuje dane z formatu pip do formatu poetry.

        Args:
            source_data: Dane w formacie pip

        Returns:
            Dane w formacie poetry
        """
        # Sprawdzamy, czy dane są w odpowiednim formacie
        if source_data.get("format") != "pip":
            raise ValueError("Dane źródłowe nie są w formacie pip")

        # Pobieramy opcje
        project_name = self.options.get("project_name", "myproject")
        version = self.options.get("version", "0.1.0")

        # Konwertujemy dane
        return PoetrySchema.convert_from_pip(source_data, project_name, version)

    def write_target(self, target_data: Dict[str, Any]) -> Path:
        """
        Zapisuje dane do pliku pyproject.toml.

        Args:
            target_data: Dane w formacie poetry

        Returns:
            Ścieżka do zapisanego pliku
        """
        if self.target_file is None:
            # Jeśli nie podano ścieżki do pliku docelowego, tworzymy ją na podstawie pliku źródłowego
            if self.source_file is None:
                raise ValueError(
                    "Nie podano ścieżki do pliku docelowego ani źródłowego"
                )

            source_path = Path(self.source_file)
            self.target_file = source_path.parent / "pyproject.toml"

        return PoetrySchema.write_pyproject_toml(target_data, self.target_file)

    def execute(self) -> Path:
        """
        Wykonuje pełny proces konwersji.

        Returns:
            Ścieżka do zapisanego pliku
        """
        # Odczytujemy plik źródłowy
        source_data = self.read_source()

        # Konwertujemy dane
        target_data = self.convert(source_data)

        # Zapisujemy dane do pliku docelowego
        return self.write_target(target_data)



"""
Moduł zawierający implementacje konwerterów dla różnych formatów pakietów.
"""

from spectomate.converters.conda_to_pip import CondaToPipConverter

# Importujemy wszystkie konwertery, aby zarejestrowały się w ConverterRegistry
from spectomate.converters.pip_to_conda import PipToCondaConverter
from spectomate.converters.pip_to_poetry import PipToPoetryConverter

# Tymczasowo usunięto import nieistniejących konwerterów
# from spectomate.converters.pip_to_pipenv import PipToPipenvConverter
# from spectomate.converters.pip_to_pdm import PipToPdmConverter
# from spectomate.converters.poetry_to_pip import PoetryToPipConverter
# from spectomate.converters.poetry_to_conda import PoetryToCondaConverter

# Importujemy zewnętrzne konwertery
try:
    from spectomate.converters.external_converters import (
        ExternalDephellConverter,
        ExternalPoetry2CondaConverter,
        ExternalReq2TomlConverter,
    )
except ImportError:
    pass  # Zewnętrzne konwertery są opcjonalne



"""
Moduł zawierający definicje schematów dla różnych formatów pakietów.
"""

from spectomate.schemas.conda_schema import CondaSchema
from spectomate.schemas.pip_schema import PipSchema
from spectomate.schemas.poetry_schema import PoetrySchema

# Tymczasowo usunięto import PipenvSchema, ponieważ klasa nie jest jeszcze zaimplementowana
# from spectomate.schemas.pipenv_schema import PipenvSchema
